from colorama import Fore, Back, Style


def merge(color_string):
    """ Given a string of colorama ANSI, merge them if you can. """
    return color_string.replace("m\x1b[", ";")


class Boxes(object):
    """ Boxes makes nice ASCII graphic menus

    Example:

        c1 = merge(Style.BRIGHT + Fore.CYAN)
        c2 = merge(Style.NORMAL + Fore.CYAN)

        # Set the size of the box to 40  (Important!)
        box = Boxes(40, color=c1)

        self.queue_game.put(box.top())
        # format string "{0:^40}".format("Text Centered")
        # This centers the text into the size of the box.

        self.queue_game.put(box.row(c1 + "{0:^40}".format("Scripts")))

        # this draws the middle line.  (The line that goes across, but is
        # connected on both sides to the one above it).
        self.queue_game.put(box.middle())


        def menu_item(ch, desc):
            # Again we want the final string to be 40!
            # c1 and c2 (below) take 0 space.
            # We have " " + ch + " " + "-" + " " .. which equals 5
            # So desc needs {:35} to pad out string out to 40 print chars.

            row = " {0}{1} {2}-{3} {4:35}".format(c1, ch, c2, c1, desc)
            # self.queue_game.put(
            #     " " + c1 + ch + c2 + " - " + c1 + desc + self.nl
            # )
            self.queue_game.put(box.row(row))

        menu_item("1", "Ports (Trades between two sectors)")
        menu_item("2", "Explore (Strange new sectors)")
        menu_item("3", "Space... the final frontier...")
        menu_item("X", "eXit")
        self.queue_game.put(box.bottom())

    """

    box_styles = (
        {
            "tl": "\xc9",
            "tr": "\xbb",
            "top": "\xcd",
            "side": "\xba",
            "bl": "\xc8",
            "br": "\xbc",
            "ml": "\xcc",
            "mr": "\xb9",
        },
    )

    def __init__(self, size: int, color="", style=0):
        """ Construct box

        size is width of text inside the box.
        (so actual width is size + 2).

        color = Style of color to use.  
        If color is blank, we don't use the RESET_ALL.

        style=0  Is double lines.  
        FUTURE:  Support all line types.
        """
        self.size = size
        self.style = 0  # style
        self.color = color

        # default colors
        # self.c = merge(Style.BRIGHT + Fore.WHITE + Back.BLUE)
        # self.cp = merge(Style.BRIGHT + Fore.YELLOW + Back.BLUE)

        # useful consts
        self.r = Style.RESET_ALL
        self.nl = "\n\r"
        # self.bsb = "\b \b"

    def top(self):
        """ Output the TOP line. """
        if self.color == "":
            c = ""
            r = ""
        else:
            c = self.color
            r = self.r
        s = self.box_styles[self.style]
        return self.nl + c + s["tl"] + s["top"] * self.size + s["tr"] + r + self.nl

    def middle(self):
        """ Output MIDDLE line.

        This solid line goes completely across.
        The edges connect to the top and bottom.
        """
        if self.color == "":
            c = ""
            r = ""
        else:
            c = self.color
            r = self.r
        s = self.box_styles[self.style]
        return c + s["ml"] + s["top"] * self.size + s["mr"] + r + self.nl

    def row(self, line):
        """ Output content line.

        NOTE:  the line length must match the box size.
        Use "{0:30}".format(content) to match size of 30.
        """
        if self.color == "":
            c = ""
            r = ""
        else:
            c = self.color
            r = self.r
        s = self.box_styles[self.style]
        return c + "{0}{1}{2}".format(s["side"], r + line + c, s["side"]) + r + self.nl

    def bottom(self):
        """ Output BOTTOM line. """
        if self.color == "":
            c = ""
            r = ""
        else:
            c = self.color
            r = self.r
        s = self.box_styles[self.style]
        return c + s["bl"] + s["top"] * self.size + s["br"] + r + self.nl

